@@ -1,6 +1,6 @@
 #include "gff_utils.h"
 
-GHash<GeneInfo> gene_ids;
+GHash<GeneInfo*> gene_ids;
 
 bool verbose=false; //same with GffReader::showWarnings and GffLoader::beVserbose
 bool debugMode=false;
@@ -14,9 +14,13 @@ int wPadding = 0; //padding for -w option
 FILE* f_x=NULL; //writing fasta with spliced CDS
 FILE* f_y=NULL; //wrting fasta with translated CDS
 
+FILE* f_j=NULL; //wrting junctions (intron coordinates)
 
 int maxintron=999000000;
 
+ID_Flt_Type IDflt=idFlt_None;
+
+bool TFilters=false;
 bool wCDSonly=false;
 bool wNConly=false;
 int minLen=0; //minimum transcript length
@@ -38,13 +42,19 @@ bool fullCDSonly=false; // starts with START, ends with STOP codon
 bool multiExon=false;
 bool writeExonSegs=false;
 char* tracklabel=NULL;
+/*
 char* rfltGSeq=NULL;
 char rfltStrand=0;
 uint rfltStart=0;
-uint rfltEnd=MAX_UINT;
+uint rfltEnd=MAX_UINT;*/
+GRangeParser* fltRange=NULL;
+
+GRangeParser* fltJunction=NULL;
+
 bool rfltWithin=false; //check for full containment within given range
 bool addDescr=false;
 
+bool wfaNoCDS=false;
 
 bool fmtGFF3=true; //default output: GFF3
 //other formats only make sense in transcriptOnly mode
@@ -57,9 +67,12 @@ GffPrintMode exonPrinting=pgffAny;
 
 GFastaDb gfasta;
 
-GHash<SeqInfo> seqinfo;
+GHash<SeqInfo*> seqinfo;
 GVec<CTableField> tableCols;
-GHash<RefTran> reftbl;
+GHash<RefTran*> reftbl;
+GStrSet<> fltIDs;
+
+GStrSet<> attrList;
 
 GHash<int> isoCounter; //counts the valid isoforms
 
@@ -158,7 +171,6 @@ int cmpRedundant(GffObj& a, GffObj& b) {
    else return (a.exons.Count()>b.exons.Count())? 1: -1;
 }
 
-
 bool tMatch(GffObj& a, GffObj& b) {
   //strict intron chain match, or single-exon perfect match
   int imax=a.exons.Count()-1;
@@ -274,6 +286,8 @@ int adjust_stopcodon(GffObj& gffrec, int adj, GList<GSeg>* seglst) {
 
 void printTableData(FILE* f, GffObj& g, bool inFasta) {
  //using attribute list in tableCols
+	const int DBUF_LEN=1024; //there should not be attribute values larger than 1K!
+	char dbuf[DBUF_LEN];
 	char* av=NULL;
 	for(int i=0;i<tableCols.Count();i++) {
 		if (i>0 || inFasta) {
@@ -283,7 +297,12 @@ void printTableData(FILE* f, GffObj& g, bool inFasta) {
 		switch(tableCols[i].type) {
 		case ctfGFF_Attr:
 			av=g.getAttr(tableCols[i].name.chars());
-			fprintf(f,"%s",av!=NULL? av : ".");
+			if (av) {
+				if (decodeChars) {
+					GffObj::decodeHexChars(dbuf, av, DBUF_LEN-1);
+					fprintf(f,"%s", dbuf);
+				} else fprintf(f,"%s",av);
+			} else fprintf(f,".");
 			break;
 		case ctfGFF_chr:
 			fprintf(f,"%s",g.getGSeqName());
@@ -353,7 +372,29 @@ void printTableData(FILE* f, GffObj& g, bool inFasta) {
 	}
 	fprintf(f,"\n");
 }
+
 bool GffLoader::validateGffRec(GffObj* gffrec) {
+	if (!checkFilters(gffrec)) {
+		if (gffrec->isTranscript()) {
+			TFilters=true;
+			if (gffrec->parent!=NULL && keepGenes) {
+			   GPVec<GffObj>& pchildren=gffrec->parent->children;
+			   for (int c=0;c<pchildren.Count();c++) {
+				   if (pchildren[c]==gffrec) {
+					   pchildren.Delete(c);
+					   break;
+				   }
+			   }
+			}
+		return false;
+		}
+		if (gffrec->isGene() && keepGenes) return true;
+		return false;
+	} //transcript rejected
+	return true;
+}
+
+bool GffLoader::checkFilters(GffObj* gffrec) {
 	if (reftbl.Count()>0) { //check if we need to reject by ref seq filter
 		GStr refname(gffrec->getRefName());
 		RefTran* rt=reftbl.Find(refname.chars());
@@ -379,44 +420,92 @@ bool GffLoader::validateGffRec(GffObj* gffrec) {
 		//GMessage("Warning: discarding %s GFF generic gene/locus container %s\n",gffrec->getID());
 		return false;
 	}
+	if (IDflt) {
+		if (fltIDs.hasKey(gffrec->getID())) {
+			if (IDflt==idFlt_Exclude) return false;
+		}
+		else if (IDflt==idFlt_Only) return false;
+	}
 	if (minLen>0 && gffrec->covlen<minLen) {
 		if (verbose)
 			GMessage("Info: %s discarded due to minimum length threshold %d\n",
 					gffrec->getID(), minLen);
     	return false;
 	}
-	if (rfltGSeq!=NULL) { //filter by gseqName
-		if (strcmp(gffrec->getGSeqName(),rfltGSeq)!=0) {
+	if (fltRange!=NULL) { //filter by gseqName
+		if (fltRange->refName!=NULL && strcmp(gffrec->getGSeqName(),fltRange->refName)!=0) {
 			return false;
 		}
-	}
-	if (rfltStrand>0 && gffrec->strand !=rfltStrand) {
-		return false;
-	}
-	//check coordinates
-	if (rfltStart!=0 || rfltEnd!=MAX_UINT) {
-		if (rfltWithin) {
-			if (gffrec->start<rfltStart || gffrec->end>rfltEnd) {
-				return false; //not within query range
+		if (fltRange->strand>0 && gffrec->strand!=fltRange->strand) {
+			return false;
+		}
+		//check coordinates
+		if (fltRange->start || fltRange->end<UINT_MAX) {
+			if (rfltWithin) {
+				if (gffrec->start<fltRange->start || gffrec->end>fltRange->end) {
+					return false; //not within query range
+				}
+			}
+			else {
+				if (gffrec->start>fltRange->end || gffrec->end<fltRange->start) {
+					return false;
+				}
 			}
 		}
-		else {
-			if (gffrec->start>rfltEnd || gffrec->end<rfltStart) {
+	}
+	if (this->attrsFilter) { //mostly relevant for transcripts and gene records
+		//remove attributes that are not in attrList
+		gffrec->removeAttrs(attrList);
+	}
+    if (gffrec->isTranscript()) {    // && TFilters) ?
+    	//these filters only apply to transcripts
+		if (multiExon && gffrec->exons.Count()<=1) {
+			return false;
+		}
+		if (wCDSonly && gffrec->CDstart==0) {
+			return false;
+		}
+		if (wNConly && gffrec->hasCDS()) return false;
+		if (fltJunction!=NULL) {
+			if (gffrec->exons.Count()<=1) return false;
+			if (fltJunction->refName!=NULL && strcmp(gffrec->getGSeqName(),fltJunction->refName)!=0) {
 				return false;
 			}
+			if (fltJunction->strand && gffrec->strand!=fltJunction->strand) {
+				return false;
+			}
+			//check coordinates
+			uint jstart=fltJunction->start;
+			uint jend=fltJunction->end;
+			if (jstart==0) jstart=jend;
+			if (jend==0)  jend=jstart;
+			if (gffrec->start>=jstart || gffrec->end<=jend) {
+						return false;
+	        }
+
+			bool noJMatch=true;
+			for (int i=0;i<gffrec->exons.Count()-1;++i) {
+				if (fltJunction->start && fltJunction->end) {
+					if (gffrec->exons[i]->end+1==fltJunction->start &&
+							gffrec->exons[i+1]->start-1==fltJunction->end)
+						{ noJMatch=false; break; }
+				} else if (fltJunction->start) { //end match not required
+					if (gffrec->exons[i]->end+1==fltJunction->start)
+						{ noJMatch=false; break; }
+				} else { //only end match required:
+					if (gffrec->exons[i+1]->start-1==fltJunction->end)
+						{ noJMatch=false; break; }
+				}
+			}
+			if (noJMatch) return false;
 		}
-	}
-	if (multiExon && gffrec->exons.Count()<=1) {
-		return false;
-	}
-	if (wCDSonly && gffrec->CDstart==0) {
-		return false;
-	}
-	if (wNConly && gffrec->hasCDS()) return false;
+
+		return process_transcript(gfasta, *gffrec);
+    } //transcript filters check
 	return true;
 }
 
-bool process_transcript(GFastaDb& gfasta, GffObj& gffrec) {
+bool GffLoader::process_transcript(GFastaDb& gfasta, GffObj& gffrec) {
  if (!gffrec.isTranscript()) return false; //shouldn't call this function unless it's a transcript
  //returns true if the transcript passed the filter
  char* gname=gffrec.getGeneName();
@@ -451,13 +540,13 @@ bool process_transcript(GFastaDb& gfasta, GffObj& gffrec) {
           }
       }
  if (gname && strcmp(gname, gffrec.getID())!=0) {
-   int* isonum=isoCounter.Find(gname);
-   if  (isonum==NULL) {
-       isonum=new int(1);
-       isoCounter.Add(gname,isonum);
-       }
-      else (*isonum)++;
-   //defline.appendfmt(" gene=%s", gname);
+	   int* isonum=isoCounter.Find(gname);
+	   if  (isonum==NULL) {
+		   //isonum=new int(1);
+		   isoCounter.Add(gname,1);
+	   }
+		else (*isonum)++;
+	   //defline.appendfmt(" gene=%s", gname);
    }
   int seqlen=0;
 
@@ -533,6 +622,7 @@ bool process_transcript(GFastaDb& gfasta, GffObj& gffrec) {
     int phaseNum=0;
   CDS_CHECK:
     uint cds_olen=0;
+    inframeStop=false;
     cdsnt=gffrec.getSpliced(faseq, true, &seqlen, NULL, &cds_olen, &seglst, adjustStop);
     //if adjustStop, seqlen has the CDS+3'UTR length, but cds_olen still has the original CDS length
     if (cdsnt!=NULL && cdsnt[0]!='\0') { //has CDS
@@ -636,40 +726,48 @@ bool process_transcript(GFastaDb& gfasta, GffObj& gffrec) {
   if (adjstop!=NULL) delete adjstop;
   */
   if (cdsnt!=NULL) { // && !inframeStop) {
+	  GStr defline(gffrec.getID(), 94);
+	  if (writeExonSegs) {
+		  defline.append(" loc:");
+		  defline.append(gffrec.getGSeqName());
+		  defline.appendfmt("(%c)",gffrec.strand);
+		  //warning: not CDS coordinates are written here, but the exon ones
+		  defline+=(int)gffrec.start;
+		  defline+=(char)'-';
+		  defline+=(int)gffrec.end;
+		  // -- here these are CDS substring coordinates on the spliced sequence:
+		  defline.append(" segs:");
+		  for (int i=0;i<seglst.Count();i++) {
+			  if (i>0) defline.append(",");
+			  defline+=(int)seglst[i].start;
+			  defline.append("-");
+			  defline+=(int)seglst[i].end;
+		  }
+	  }
 	  if (f_y!=NULL) { //CDS translation fasta output requested
 			 if (cdsaa==NULL) { //translate now if not done before
 			   cdsaa=translateDNA(cdsnt, aalen, seqlen);
 			 }
 			 if (aalen>0) {
 			   if (cdsaa[aalen-1]=='.' || cdsaa[aalen-1]=='\0') --aalen; //avoid printing the stop codon
- 			   fprintf(f_y, ">%s", gffrec.getID());
+ 			   fprintf(f_y, ">%s", defline.chars());
  			   if (fmtTable) printTableData(f_y, gffrec, true);
- 			   else fprintf(f_y, "\n");
+ 			   else {
+ 				  if (gffrec.attrs!=NULL && gffrec.attrs->Count()>0) fprintf(f_y," ");
+ 				  gffrec.printAttrs(f_y, ";", false, decodeChars, false);
+ 				  fprintf(f_y, "\n");
+ 			   }
 			   printFasta(f_y, NULL, cdsaa, aalen, StarStop);
 			 }
 	  }
 	  if (f_x!=NULL) { //CDS only
-			 GStr defline(gffrec.getID(), 94);
-			 if (writeExonSegs) {
-				  defline.append(" loc:");
-				  defline.append(gffrec.getGSeqName());
-				  defline.appendfmt("(%c)",gffrec.strand);
-				  //warning: not CDS coordinates are written here, but the exon ones
-				  defline+=(int)gffrec.start;
-				  defline+=(char)'-';
-				  defline+=(int)gffrec.end;
-				  // -- here these are CDS substring coordinates on the spliced sequence:
-				  defline.append(" segs:");
-				  for (int i=0;i<seglst.Count();i++) {
-					  if (i>0) defline.append(",");
-					  defline+=(int)seglst[i].start;
-					  defline.append("-");
-					  defline+=(int)seglst[i].end;
-					  }
-			 }
 			 fprintf(f_x, ">%s", defline.chars());
 			 if (fmtTable) printTableData(f_x, gffrec, true);
-			 else fprintf(f_x, "\n");
+			 else {
+				 if (gffrec.attrs!=NULL && gffrec.attrs->Count()>0) fprintf(f_x," ");
+				 gffrec.printAttrs(f_x, ";", false, decodeChars, false);
+				 fprintf(f_x, "\n");
+			 }
 			 printFasta(f_x, NULL, cdsnt, seqlen);
 	  }
 	  GFREE(cdsnt);
@@ -694,7 +792,7 @@ bool process_transcript(GFastaDb& gfasta, GffObj& gffrec) {
 
 	  GStr defline(gffrec.getID());
 	  if (exont!=NULL) {
-		  if (gffrec.CDstart>0) {
+		  if (!wfaNoCDS && gffrec.CDstart>0) {
 			  defline.appendfmt(" CDS=%d-%d", cds_start, cds_end);
 		  }
 		  if (writeExonSegs) {
@@ -731,7 +829,11 @@ bool process_transcript(GFastaDb& gfasta, GffObj& gffrec) {
 
 		  fprintf(f_w, ">%s", defline.chars());
 		  if (fmtTable) printTableData(f_w, gffrec, true);
-		    else fprintf(f_w, "\n");
+		    else {
+		    	if (gffrec.attrs!=NULL && gffrec.attrs->Count()>0) fprintf(f_w," ");
+		    	gffrec.printAttrs(f_w, ";", false, decodeChars, false);
+		    	fprintf(f_w, "\n");
+		    }
 		  printFasta(f_w, NULL, exont, seqlen);
 		  GFREE(exont);
 	  }
@@ -749,8 +851,6 @@ GTData::GTData(GffObj* t, GenomicSeqData* gd):rna(t),gdata(gd), locus(NULL), rep
  	   gdata->tdata.Add(this);
 }
 
-
-
 bool GffLoader::unsplContained(GffObj& ti, GffObj&  tj) {
  //returns true only if ti (which MUST be single-exon) is "almost" contained in any of tj's exons
  //but it does not cross any intron-exon boundary of tj
@@ -1169,9 +1269,9 @@ void collectLocusData(GList<GenomicSeqData>& ref_data, bool covInfo) {
 		GenomicSeqData* gdata=ref_data[g];
 		for (int l=0;l<gdata->loci.Count();l++) {
 			GffLocus& loc=*(gdata->loci[l]);
-			GHash<int> gnames(true); //gene names in this locus
+			GHash<int> gnames; //gene names in this locus
 			//GHash<int> geneids(true); //Entrez GeneID: numbers
-			GHash<int> geneids(true);
+			GHash<int> geneids;
 			int fstrand=0,rstrand=0,ustrand=0;
 			for (int i=0;i<loc.rnas.Count();i++) {
 				GffObj& t=*(loc.rnas[i]);
@@ -1185,11 +1285,14 @@ void collectLocusData(GList<GenomicSeqData>& ref_data, bool covInfo) {
 					gname.upper();
 					int* prevg=gnames.Find(gname.chars());
 					if (prevg!=NULL) (*prevg)++;
-					else gnames.Add(gname, new int(1));
+					else gnames.Add(gname.chars(), 1);
 				}
 				GStr geneid(t.getGeneID());
-				if (!geneid.is_empty())
-					geneids.Add(geneid.chars());
+				if (!geneid.is_empty()) {
+					int* prevg=gnames.Find(geneid.chars());
+					if (prevg!=NULL) (*prevg)++;
+					geneids.Add(geneid.chars(), 1);
+				}
 				//parse GeneID xrefs, if any (RefSeq):
 				/*
 				GStr xrefs(t.getAttr("xrefs"));
@@ -1220,11 +1323,13 @@ void collectLocusData(GList<GenomicSeqData>& ref_data, bool covInfo) {
 						gname.upper();
 						int* prevg=gnames.Find(gname.chars());
 						if (prevg!=NULL) (*prevg)++;
-						else gnames.Add(gname, new int(1));
+						else gnames.Add(gname, 1);
 					}
 					GStr geneid(nt.getID());
 					if (!geneid.is_empty()) {
-						geneids.Add(geneid.chars(), new int(1));
+						int* prevg=gnames.Find(geneid.chars());
+						if (prevg!=NULL) (*prevg)++;
+						geneids.Add(geneid.chars(),1);
 					}
 				}
 				//parse GeneID xrefs, if any (RefSeq):
@@ -1258,18 +1363,18 @@ void collectLocusData(GList<GenomicSeqData>& ref_data, bool covInfo) {
 			loc.locus_num=locus_num;
 			if (gnames.Count()>0) { //collect all gene names associated to this locus
 				gnames.startIterate();
-				int* gfreq=NULL;
-				char* key=NULL;
-				while ((gfreq=gnames.NextData(key))!=NULL) {
-					loc.gene_names.AddIfNew(new CGeneSym(key,*gfreq));
+				int gfreq=0;
+				const char* key=NULL;
+				while ((key=gnames.Next(gfreq))!=NULL) {
+					loc.gene_names.AddIfNew(new CGeneSym(key, gfreq));
 				}
 			} //added collected gene_names
 			if (geneids.Count()>0) { //collect all GeneIDs names associated to this locus
 				geneids.startIterate();
-				int* gfreq=NULL;
-				char* key=NULL;
-				while ((gfreq=geneids.NextData(key))!=NULL) {
-					loc.gene_ids.AddIfNew(new CGeneSym(key,*gfreq));
+				int gfreq=0;
+				const char* key=NULL;
+				while ((key=geneids.Next(gfreq))!=NULL) {
+					loc.gene_ids.AddIfNew(new CGeneSym(key, gfreq));
 				}
 			}
 		} //for each locus
@@ -1311,6 +1416,21 @@ GenomicSeqData* getGSeqData(GList<GenomicSeqData>& seqdata, int gseq_id) {
 void warnPseudo(GffObj& m) {
 	GMessage("Info: pseudo gene/transcript record with ID=%s discarded.\n",m.getID());
 }
+void GffLoader::collectIntrons(GffObj& t) {
+	// assume t are coming grouped by chromosome and sorted by start coordinate!
+	if (intronList.jlst.Count()>0) {
+         if (t.gseq_id!=intronList.gseq_id ||
+        		 t.start>=intronList.jlst.Last()->start) {
+        	intronList.print(f_j);
+        	intronList.clear();
+         }
+         else if (t.start<intronList.last_t_start)
+        	 GError("Error collectIntrons(%s) called when last_t_start was %d\n",
+        			 t.getID(), intronList.last_t_start );
+         //add this transcript's introns
+	}
+    intronList.add(t);
+}
 
 void GffLoader::load(GList<GenomicSeqData>& seqdata, GFFCommentParser* gf_parsecomment) {
 	if (f==NULL) GError("Error: GffLoader::load() cannot be called before ::openFile()!\n");
@@ -1326,6 +1446,7 @@ void GffLoader::load(GList<GenomicSeqData>& seqdata, GFFCommentParser* gf_parsec
 	gffr->keepGenes(keepGenes);
 	gffr->setIgnoreLocus(ignoreLocus);
 	gffr->setRefAlphaSorted(this->sortRefsAlpha);
+	gffr->procEnsemblID(this->ensemblProc);
 	if (keepGff3Comments && gf_parsecomment!=NULL) gffr->setCommentParser(gf_parsecomment);
     int outcounter=0;
 	if (streamIn) { //this will ignore any clustering options
@@ -1335,22 +1456,27 @@ void GffLoader::load(GList<GenomicSeqData>& seqdata, GFFCommentParser* gf_parsec
 				delete t;
 				continue;
 			}
-			if (process_transcript(gfasta, *t)) {
-				outcounter++;
-				if (f_out) {
-				  if (fmtTable)
-						printTableData(f_out, *t);
-				  else //GFF3, GTF, BED, TLF
-					t->printGxf(f_out, exonPrinting, tracklabel, NULL, decodeChars);
-				}
+
+			if (f_j!=NULL && t->isTranscript() && t->exons.Count()>1)
+				collectIntrons(*t);
+
+			outcounter++;
+			if (f_out) {
+			  if (fmtTable)
+					printTableData(f_out, *t);
+			  else //GFF3, GTF, BED, TLF
+				t->printGxf(f_out, exonPrinting, tracklabel, NULL, decodeChars);
 			}
 			delete t;
 		}
+		if (f_j && intronList.jlst.Count()>0) {
+        	intronList.print(f_j);
+        	intronList.clear();
+		}
 		delete gffr;
 		return;
 	}
 
-
 	gffr->readAll();
 	GVec<int> pseudoFeatureIds; //feature type: pseudo*
 	GVec<int> pseudoAttrIds;  // attribute: [is]pseudo*=true/yes/1
@@ -1438,9 +1564,12 @@ void GffLoader::load(GList<GenomicSeqData>& seqdata, GFFCommentParser* gf_parsec
 			m->subftype_id=gff_fid_exon;
 		}
 		//GList<GffObj> gfadd(false,false); -- for gf_validate()?
-		if (!validateGffRec(m)) {
+		if (!validateGffRec(m)) { //this will also apply process_transcript() CDS filters etc.
 			continue;
 		}
+		if (f_j!=NULL && m->isTranscript() && m->exons.Count()>1)
+			collectIntrons(*m);
+
 		m->isUsed(true); //so the gffreader won't destroy it
 		GenomicSeqData* gdata=getGSeqData(seqdata, m->gseq_id);
 		bool keep=placeGf(m, gdata);
@@ -1450,6 +1579,10 @@ void GffLoader::load(GList<GenomicSeqData>& seqdata, GFFCommentParser* gf_parsec
 			//GMessage("Feature %s(%d-%d) is going to be discarded..\n",m->getID(), m->start, m->end);
 		}
 	} //for each read gffObj
+	if (f_j && intronList.jlst.Count()>0) {
+    	intronList.print(f_j);
+    	intronList.clear();
+	}
 	//if (verbose) GMessage("  .. %d records from %s clustered into loci.\n", gffr->gflst.Count(), fname.chars());
 	//if (f && f!=stdin) { fclose(f); f=NULL; }
 	delete gffr;
