@@ -4,29 +4,32 @@
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
 
-#define VERSION "0.12.1"
+#define VERSION "0.12.7"
 
 #define USAGE "gffread v" VERSION ". Usage:\n\
-gffread <input_gff> [-g <genomic_seqs_fasta> | <dir>][-s <seq_info.fsize>] \n\
- [-o <outfile>] [-t <trackname>] [-r [[<strand>]<chr>:]<start>..<end> [-R]]\n\
+gffread [-g <genomic_seqs_fasta> | <dir>] [-s <seq_info.fsize>] \n\
+ [-o <outfile>] [-t <trackname>] [-r [<strand>]<chr>:<start>-<end> [-R]]\n\
+ [--jmatch <chr>:<start>-<end>] [--no-pseudo] \n\
  [-CTVNJMKQAFPGUBHZWTOLE] [-w <exons.fa>] [-x <cds.fa>] [-y <tr_cds.fa>]\n\
- [-i <maxintron>] [--stream] [--bed] [--table <attrlist>] [--sort-by <ref.lst>]\n\
- \n\
+ [-j ][--ids <IDs.lst> | --nids <IDs.lst>] [--attrs <attr-list>] [-i <maxintron>]\n\
+ [--stream] [--bed | --gtf | --tlf] [--table <attrlist>] [--sort-by <ref.lst>]\n\
+ [<input_gff>] \n\n\
  Filter, convert or cluster GFF/GTF/BED records, extract the sequence of\n\
  transcripts (exon or CDS) and more.\n\
  By default (i.e. without -O) only transcripts are processed, discarding any\n\
  other non-transcript features. Default output is a simplified GFF3 with only\n\
  the basic attributes.\n\
  \n\
- <input_gff> is a GFF file, use '-' for stdin\n\
- \n\
 Options:\n\
+ --ids discard records/transcripts if their IDs are not listed in <IDs.lst>\n\
+ --nids discard records/transcripts if their IDs are listed in <IDs.lst>\n\
  -i   discard transcripts having an intron larger than <maxintron>\n\
  -l   discard transcripts shorter than <minlen> bases\n\
  -r   only show transcripts overlapping coordinate range <start>..<end>\n\
       (on chromosome/contig <chr>, strand <strand> if provided)\n\
  -R   for -r option, discard all transcripts that are not fully \n\
       contained within the given range\n\
+ --jmatch only output transcripts matching the given junction\n\
  -U   discard single-exon transcripts\n\
  -C   coding only: discard mRNAs that have no CDS features\n\
  --nc non-coding only: discard mRNAs that have CDS features\n\
@@ -42,11 +45,13 @@ Sorting: (by default, chromosomes are kept in the order they were found)\n\
  --sort-by : sort the reference sequences by the order in which their\n\
       names are given in the <refseq.lst> file\n\
 Misc options: \n\
- -F   preserve all GFF attributes (for non-exon features)\n\
+ -F   keep all GFF attributes (for non-exon features)\n\
  --keep-exon-attrs : for -F option, do not attempt to reduce redundant\n\
       exon/CDS attributes\n\
  -G   do not keep exon attributes, move them to the transcript feature\n\
       (for GFF3 output)\n\
+ --attrs <attr-list> only output the GTF/GFF attributes listed in <attr-list>\n\
+    which is a comma delimited list of attribute names to\n\
  --keep-genes : in transcript-only mode (default), also preserve gene records\n\
  --keep-comments: for GFF3 input/output, try to preserve comments\n\
  -O   process other non-transcript GFF records (by default non-transcript\n\
@@ -78,7 +83,7 @@ Misc options: \n\
            ((no sorting, exons must be grouped by transcript in the input data)\n\
 Clustering:\n\
  -M/--merge : cluster the input transcripts into loci, discarding\n\
-      \"duplicated\" transcripts (those with the same exact introns\n\
+      \"redundant\" transcripts (those with the same exact introns\n\
       and fully contained or equal boundaries)\n\
  -d <dupinfo> : for -M option, write duplication info to file <dupinfo>\n\
  --cluster-only: same as -M/--merge but without discarding any of the\n\
@@ -102,24 +107,26 @@ Output options:\n\
  -g   full path to a multi-fasta file with the genomic sequences\n\
       for all input mappings, OR a directory with single-fasta files\n\
       (one per genomic sequence, with file names matching sequence names)\n\
+ -j    output the junctions and the corresponding transcripts\n\
  -w    write a fasta file with spliced exons for each transcript\n\
  --w-add <N> for the -w option, extract additional <N> bases\n\
        both upstream and downstream of the transcript boundaries\n\
+ --w-nocds for -w, disable the output of CDS info in the FASTA file\n\
  -x    write a fasta file with spliced CDS for each GFF transcript\n\
  -y    write a protein fasta file with the translation of CDS for each record\n\
- -W    for -w and -x options, write in the FASTA defline all the exon\n\
+ -W    for -w, -x and -y options, write in the FASTA defline all the exon\n\
        coordinates projected onto the spliced sequence;\n\
  -S    for -y option, use '*' instead of '.' as stop codon translation\n\
- -L    Ensembl GTF to GFF3 conversion (implies -F; should be used with -m)\n\
+ -L    Ensembl GTF to GFF3 conversion, adds version to IDs\n\
  -m    <chr_replace> is a name mapping table for converting reference \n\
        sequence names, having this 2-column format:\n\
        <original_ref_ID> <new_ref_ID>\n\
  -t    use <trackname> in the 2nd column of each GFF/GTF output line\n\
- -o    write the records into <outfile> instead of stdout\n\
+ -o    write the output records into <outfile> instead of stdout\n\
  -T    main output will be GTF instead of GFF3\n\
  --bed output records in BED format instead of default GFF3\n\
  --tlf output \"transcript line format\" which is like GFF\n\
-       but exons, CDS features and related data are stored as GFF \n\
+       but with exons and CDS related features stored as GFF \n\
        attributes in the transcript feature line, like this:\n\
          exoncount=N;exons=<exons>;CDSphase=<N>;CDS=<CDScoords> \n\
        <exons> is a comma-delimited list of exon_start-exon_end coordinates;\n\
@@ -136,61 +143,11 @@ Output options:\n\
        problems with the given GFF/GTF records\n\
 "
 
-/*
-FILE* ffasta=NULL;
-FILE* f_in=NULL;
-FILE* f_out=NULL;
-FILE* f_w=NULL; //writing fasta with spliced exons (transcripts)
-int wPadding = 0; //padding for -w option
-FILE* f_x=NULL; //writing fasta with spliced CDS
-FILE* f_y=NULL; //wrting fasta with translated CDS
-
-bool wCDSonly=false;
-bool wNConly=false;
-int minLen=0; //minimum transcript length
-bool validCDSonly=false; // translation with no in-frame STOP
-bool bothStrands=false; //for single-exon mRNA validation, check the other strand too
-bool altPhases=false; //if original phase fails translation validation,
-                     //try the other 2 phases until one makes it
-bool addCDSattrs=false;
-bool add_hasCDS=false;
-//bool streamIn=false; // --stream option
-bool adjustStop=false; //automatic adjust the CDS stop coordinate
-bool covInfo=false; // --cov-info : only report genome coverage
-GStr tableFormat; //list of "attributes" to print in tab delimited format
-bool spliceCheck=false; //only known splice-sites
-bool decodeChars=false; //decode url-encoded chars in attrs (-D)
-bool StarStop=false; //use * instead of . for stop codon translation
-bool fullCDSonly=false; // starts with START, ends with STOP codon
-*/
-
 GStr sortBy; //file name with chromosomes listed in the desired order
 
 bool BEDinput=false;
 bool TLFinput=false;
 
-//--- output options
-/*
-extern bool fmtGFF3=true; //default output: GFF3
-//other formats only make sense in transcriptOnly mode
-extern bool fmtGTF=false;
-extern bool fmtBED=false;
-extern bool fmtTLF=false;
-extern bool fmtTable=false;
-
-
-bool multiExon=false;
-bool writeExonSegs=false;
-char* tracklabel=NULL;
-char* rfltGSeq=NULL;
-char rfltStrand=0;
-uint rfltStart=0;
-uint rfltEnd=MAX_UINT;
-bool rfltWithin=false; //check for full containment within given range
-bool addDescr=false;
-
-*/
-
 //bool protmap=false;
 //int maxintron=999000000;
 //bool mergeCloseExons=false;
@@ -200,7 +157,22 @@ GffLoader gffloader;
 
 GList<GenomicSeqData> g_data(true,true,true); //list of GFF records by genomic seq
 
-void loadSeqInfo(FILE* f, GHash<SeqInfo> &si) {
+void loadIDlist(FILE* f, GStrSet<> & idhash) {
+  GLineReader fr(f);
+  while (!fr.isEof()) {
+      char* line=fr.getLine();
+      if (line==NULL) break;
+      if (line[0]=='#') continue; //skip comments
+      GDynArray<char*> ids;
+      strsplit(line, ids);
+      for (uint i=0;i<ids.Count();i++) {
+    	  if (strlen(ids[i])>0)
+    		  idhash.Add(ids[i]);
+      }
+  }
+}
+
+void loadSeqInfo(FILE* f, GHash<SeqInfo*> &si) {
   GLineReader fr(f);
   while (!fr.isEof()) {
       char* line=fr.getLine();
@@ -230,23 +202,33 @@ void loadSeqInfo(FILE* f, GHash<SeqInfo> &si) {
       } //while lines
 }
 
+void getAttrList(GStr& s) {
+	 if (s.is_empty()) return;
+	 s.startTokenize(",;:", tkCharSet);
+	 GStr w;
+	 while (s.nextToken(w)) {
+		  if (w.length()>0)
+    	    attrList.Add(w.chars());
+	 }
+}
+
 void setTableFormat(GStr& s) {
 	 if (s.is_empty()) return;
 	 GHash<ETableFieldType> specialFields;
-	 specialFields.Add("chr", new ETableFieldType(ctfGFF_chr));
-	 specialFields.Add("id", new ETableFieldType(ctfGFF_ID));
-	 specialFields.Add("geneid", new ETableFieldType(ctfGFF_geneID));
-	 specialFields.Add("genename", new ETableFieldType(ctfGFF_geneName));
-	 specialFields.Add("parent", new ETableFieldType(ctfGFF_Parent));
-	 specialFields.Add("feature", new ETableFieldType(ctfGFF_feature));
-	 specialFields.Add("start", new ETableFieldType(ctfGFF_start));
-	 specialFields.Add("end", new ETableFieldType(ctfGFF_end));
-	 specialFields.Add("strand", new ETableFieldType(ctfGFF_strand));
-	 specialFields.Add("numexons", new ETableFieldType(ctfGFF_numexons));
-	 specialFields.Add("exons", new ETableFieldType(ctfGFF_exons));
-	 specialFields.Add("cds", new ETableFieldType(ctfGFF_cds));
-	 specialFields.Add("covlen", new ETableFieldType(ctfGFF_covlen));
-	 specialFields.Add("cdslen", new ETableFieldType(ctfGFF_cdslen));
+	 specialFields.Add("chr", ctfGFF_chr);
+	 specialFields.Add("id", ctfGFF_ID);
+	 specialFields.Add("geneid", ctfGFF_geneID);
+	 specialFields.Add("genename", ctfGFF_geneName);
+	 specialFields.Add("parent", ctfGFF_Parent);
+	 specialFields.Add("feature", ctfGFF_feature);
+	 specialFields.Add("start", ctfGFF_start);
+	 specialFields.Add("end", ctfGFF_end);
+	 specialFields.Add("strand", ctfGFF_strand);
+	 specialFields.Add("numexons", ctfGFF_numexons);
+	 specialFields.Add("exons", ctfGFF_exons);
+	 specialFields.Add("cds", ctfGFF_cds);
+	 specialFields.Add("covlen", ctfGFF_covlen);
+	 specialFields.Add("cdslen", ctfGFF_cdslen);
 
 	 s.startTokenize(" ,;.:", tkCharSet);
 	 GStr w;
@@ -272,11 +254,6 @@ void setTableFormat(GStr& s) {
     	  tableCols.Add(tcol);
     	  continue;
       }
-      if (w=="geneID" || w=="gene_id") {
-    	  CTableField tcol(ctfGFF_geneID);
-    	  tableCols.Add(tcol);
-    	  continue;
-      }
       if (w=="Parent") {
     	  CTableField tcol(ctfGFF_Parent);
     	  tableCols.Add(tcol);
@@ -287,7 +264,7 @@ void setTableFormat(GStr& s) {
 	 }
 }
 
-void loadRefTable(FILE* f, GHash<RefTran>& rt) {
+void loadRefTable(FILE* f, GHash<RefTran*>& rt) {
   GLineReader fr(f);
   char* line=NULL;
   while ((line=fr.getLine())) {
@@ -412,20 +389,27 @@ void shutDown() {
 	seqinfo.Clear();
 	//if (faseq!=NULL) delete faseq;
 	//if (gcdb!=NULL) delete gcdb;
-	GFREE(rfltGSeq);
+	delete fltRange;
+	delete fltJunction;
 	FWCLOSE(f_out);
 	FWCLOSE(f_w);
 	FWCLOSE(f_x);
 	FWCLOSE(f_y);
+	FWCLOSE(f_j);
 }
 
 int main(int argc, char* argv[]) {
  GArgs args(argc, argv,
    "version;debug;merge;stream;adj-stop;bed;in-bed;tlf;in-tlf;cluster-only;nc;cov-info;help;"
-    "sort-alpha;keep-genes;w-add=;keep-comments;keep-exon-attrs;force-exons;t-adopt;gene2exon;"
-    "ignore-locus;no-pseudo;table=sort-by=hvOUNHPWCVJMKQYTDARSZFGLEBm:g:i:r:s:l:t:o:w:x:y:d:");
+    "sort-alpha;keep-genes;w-nocds;attrs=;w-add=;ids=;nids=;jmatch=;gtf;keep-comments;keep-exon-attrs;force-exons;t-adopt;gene2exon;"
+    "ignore-locus;no-pseudo;table=sort-by=hvOUNHPWCVJMKQYTDARSZFGLEBm:g:i:r:s:l:t:o:w:x:y:j:d:");
  args.printError(USAGE, true);
- if (args.getOpt('h') || args.getOpt("help")) {
+ int numfiles = args.startNonOpt();
+ if (args.getOpt("version")) {
+    printf(VERSION"\n");
+    exit(0);
+ }
+ if (args.getOpt('h') || args.getOpt("help") || ( numfiles==0 && !haveStdInput())) {
     GMessage("%s",USAGE);
     exit(1);
  }
@@ -448,9 +432,9 @@ int main(int argc, char* argv[]) {
  if (adjustStop) addCDSattrs=true;
  validCDSonly=(args.getOpt('V')!=NULL);
  altPhases=(args.getOpt('H')!=NULL);
- fmtGTF=(args.getOpt('T')!=NULL); //switch output format to GTF
- fmtBED=(args.getOpt("bed")!=NULL);
- fmtTLF=(args.getOpt("tlf")!=NULL);
+ fmtGTF=(args.getOpt('T')!=NULL || args.getOpt("gtf")!=NULL); //switch output format to GTF
+ fmtBED=(args.getOpt("bed")!=NULL); //BED output
+ fmtTLF=(args.getOpt("tlf")!=NULL); //TLF output
  if (fmtGTF || fmtBED || fmtTLF) {
 	 if (!gffloader.transcriptsOnly) {
 		 GMessage("Error: option -O is only supported with GFF3 output");
@@ -509,10 +493,6 @@ int main(int argc, char* argv[]) {
      fprintf(stderr, "Command line was:\n");
      args.printCmdLine(stderr);
      }
- if (args.getOpt("version")) {
-  GMessage(VERSION"\n");
-  exit(0);
- }
  gffloader.fullAttributes=(args.getOpt('F')!=NULL);
  gffloader.keep_AllExonAttrs=(args.getOpt("keep-exon-attrs")!=NULL);
  if (gffloader.keep_AllExonAttrs && !gffloader.fullAttributes) {
@@ -529,8 +509,8 @@ int main(int argc, char* argv[]) {
 	 gffloader.gatherExonAttrs=true;
 	 gffloader.fullAttributes=true;
  }
- ensembl_convert=(args.getOpt('L')!=NULL);
- if (ensembl_convert) {
+ gffloader.ensemblProc=(args.getOpt('L')!=NULL);
+ if (gffloader.ensemblProc) {
     gffloader.fullAttributes=true;
     gffloader.gatherExonAttrs=false;
     //sortByLoc=true;
@@ -558,7 +538,7 @@ int main(int argc, char* argv[]) {
  if (!s.is_empty()) maxintron=s.asInt();
  s=args.getOpt('l');
  if (!s.is_empty()) minLen=s.asInt();
-
+ TFilters=(multiExon || wCDSonly || wNConly); //TODO: all transcript filters should be included here through validateGffRec()
  FILE* f_repl=NULL; //duplicate/collapsing info output file
  s=args.getOpt('d');
  if (!s.is_empty()) {
@@ -569,45 +549,29 @@ int main(int argc, char* argv[]) {
    }
  }
 
- rfltWithin=(args.getOpt('R')!=NULL);
- s=args.getOpt('r');
+ s=args.getOpt("attrs");
  if (!s.is_empty()) {
-   s.trim();
-   if (s[0]=='+' || s[0]=='-') {
-     rfltStrand=s[0];
-     s.cut(0,1);
-     }
-   int isep=s.index(':');
-   if (isep>0) { //gseq name given
-      if (rfltStrand==0 && (s[isep-1]=='+' || s[isep-1]=='-')) {
-        isep--;
-        rfltStrand=s[isep];
-        s.cut(isep,1);
-        }
-      if (isep>0)
-          rfltGSeq=Gstrdup((s.substr(0,isep)).chars());
-      s.cut(0,isep+1);
-      }
-   GStr gsend;
-   char slast=s[s.length()-1];
-   if (rfltStrand==0 && (slast=='+' || slast=='-')) {
-      s.chomp(slast);
-      rfltStrand=slast;
-      }
-   if (s.index("..")>=0) gsend=s.split("..");
-                    else gsend=s.split('-');
-   if (!s.is_empty()) rfltStart=(uint)s.asInt();
-   if (!gsend.is_empty()) {
-      rfltEnd=(uint)gsend.asInt();
-      if (rfltEnd==0) rfltEnd=MAX_UINT;
-      }
-   } //gseq/range filtering
- else {
+	 getAttrList(s);
+	 gffloader.attrsFilter=(attrList.Count()>1);
+	 gffloader.fullAttributes=true;
+ }
+ rfltWithin=(args.getOpt('R')!=NULL);
+ char* sz=args.getOpt('r');
+ if (sz) {
+	fltRange=new GRangeParser(sz);
+ 	if (fltRange->end==0) //end coordinate not given
+ 		fltRange->end=UINT_MAX;
+ } else {
    if (rfltWithin)
      GError("Error: option -R requires -r!\n");
-   //if (rfltWholeTranscript)
-   //  GError("Error: option -P requires -r!\n");
-   }
+ }
+ sz=args.getOpt("jmatch");
+ if (sz) {
+	//TODO: check if this is a file?
+	fltJunction=new GRangeParser(sz);
+	if (fltJunction->strand=='.') fltJunction->strand=0;
+ } //gseq/range filtering
+
  s=args.getOpt('m');
  if (!s.is_empty()) {
    FILE* ft=fopen(s,"r");
@@ -621,8 +585,25 @@ int main(int argc, char* argv[]) {
    if (fsize==NULL) GError("Error opening info file: %s\n",s.chars());
    loadSeqInfo(fsize, seqinfo);
    fclose(fsize);
+ }
+ s=args.getOpt("ids");
+ if (s.is_empty()) {
+	 s=args.getOpt("nids");
+	 if (!s.is_empty())
+		 IDflt=idFlt_Exclude;
+ } else {
+	 IDflt=idFlt_Only;
+ }
+ if (!s.is_empty()) {
+   FILE* f=fopen(s,"r");
+   if (f==NULL) GError("Error opening ID list file: %s\n",s.chars());
+   loadIDlist(f, fltIDs);
+   if (fltIDs.Count()==0) {
+	   GMessage("Warning: no IDs were loaded from file %s\n", s.chars());
+	   IDflt=idFlt_None;
    }
-
+   fclose(f);
+ }
  openfw(f_out, args, 'o');
  //if (f_out==NULL) f_out=stdout;
  if (gfasta.fastaPath==NULL && (validCDSonly || spliceCheck || args.getOpt('w')!=NULL || args.getOpt('x')!=NULL || args.getOpt('y')!=NULL))
@@ -630,19 +611,22 @@ int main(int argc, char* argv[]) {
  openfw(f_w, args, 'w');
  openfw(f_x, args, 'x');
  openfw(f_y, args, 'y');
+ openfw(f_j, args, 'j');
  s=args.getOpt("w-add");
  if (!s.is_empty()) {
 	 if (f_w==NULL) GError("Error: --w-add option requires -w option!\n");
 	 wPadding=s.asInt();
  }
 
- if (f_out==NULL && f_w==NULL && f_x==NULL && f_y==NULL && !covInfo)
+ if (f_w!=NULL && args.getOpt("w-nocds"))
+	 wfaNoCDS=true;
+
+ if (f_out==NULL && f_w==NULL && f_x==NULL && f_y==NULL && f_j==NULL && !covInfo)
 	 f_out=stdout;
 
  //if (f_y!=NULL || f_x!=NULL) wCDSonly=true;
  //useBadCDS=useBadCDS || (fgtfok==NULL && fgtfbad==NULL && f_y==NULL && f_x==NULL);
 
- int numfiles = args.startNonOpt();
  //GList<GffObj> gfkept(false,true); //unsorted, free items on delete
  int out_counter=0; //number of records printed
 
@@ -726,8 +710,8 @@ int main(int argc, char* argv[]) {
        bool firstLocusPrint=true;
        GffLocus& loc=*(gdata->loci[l]);
        //check all non-replaced transcripts in this locus:
-       int numvalid=0;
-       int idxfirstvalid=-1;
+       //int numvalid=0;
+       //int idxfirstvalid=-1;
        for (int i=0;i<loc.rnas.Count();i++) {
          GffObj& t=*(loc.rnas[i]);
          GTData* tdata=(GTData*)(t.uptr);
@@ -752,15 +736,16 @@ int main(int argc, char* argv[]) {
          //restore strand for dOvlSET
          char orig_strand=T_OSTRAND(t.udata);
          if (orig_strand!=0) t.strand=orig_strand;
-
+         /* -- transcripts are filtered upon loading
          if (process_transcript(gfasta, t)) {
              numvalid++;
              if (idxfirstvalid<0) idxfirstvalid=i;
          }
+         */
        } //for each transcript
 
        int rnas_i=0;
-       if (idxfirstvalid>=0) rnas_i=idxfirstvalid;
+       //if (idxfirstvalid>=0) rnas_i=idxfirstvalid;
        int gfs_i=0;
        if (f_out) {
            GStr locname("RLOC_");
@@ -775,7 +760,8 @@ int main(int argc, char* argv[]) {
 					   if (firstGff3Print) { printGff3Header(f_out, args);firstGff3Print=false; }
 					   if (firstGSeqHeader) { printGSeqHeader(f_out, gdata); firstGSeqHeader=false; }
 					   if (firstLocusPrint) {
-						   loc.print(f_out, idxfirstvalid, locname, loctrack);
+						   //loc.print(f_out, idxfirstvalid, locname, loctrack);
+						   loc.print(f_out, 0, locname, loctrack);
 						   firstLocusPrint=false;
 					   }
 					   printGffObj(f_out, loc.gfs[gfs_i], locname, exonPrinting, out_counter);
@@ -789,7 +775,8 @@ int main(int argc, char* argv[]) {
 					     if (firstGff3Print) { printGff3Header(f_out, args); firstGff3Print=false; }
 					     if (firstGSeqHeader) { printGSeqHeader(f_out, gdata); firstGSeqHeader=false; }
 					     if (firstLocusPrint) {
-					    	 loc.print(f_out, idxfirstvalid, locname, loctrack);
+					    	 //loc.print(f_out, idxfirstvalid, locname, loctrack);
+					    	 loc.print(f_out, 0, locname, loctrack);
 					    	 firstLocusPrint=false;
 					     }
 				       }
@@ -804,7 +791,7 @@ int main(int argc, char* argv[]) {
    } //if Clustering enabled
   else { //no clustering
    //not grouped into loci, print the rnas with their parents, if any
-   int numvalid=0;
+   //int numvalid=0;
    for (int g=0;g<g_data.Count();g++) {
      GenomicSeqData* gdata=g_data[g];
      bool firstGSeqHeader=fmtGFF3;
@@ -815,6 +802,8 @@ int main(int argc, char* argv[]) {
          //print other non-transcript (gene?) feature that might be there before t
            while (gfs_i<gdata->gfs.Count() && gdata->gfs[gfs_i]->start<=t.start) {
              GffObj& gfst=*(gdata->gfs[gfs_i]);
+             if (TFilters && gfst.isGene() && gfst.children.Count()==0) // gene with no children left, skip it if filters were applied
+            	 { ++gfs_i; continue; }
              if T_PRINTABLE(gfst.udata) { //never printed
                T_NO_PRINT(gfst.udata);
                if (fmtGFF3) {
@@ -829,8 +818,8 @@ int main(int argc, char* argv[]) {
         }
         GTData* tdata=(GTData*)(t.uptr);
         if (tdata->replaced_by!=NULL) continue;
-        if (process_transcript(gfasta, t)) {
-           numvalid++;
+        //if (process_transcript(gfasta, t)) {
+        //   numvalid++;
            if (f_out && T_PRINTABLE(t.udata) ) {
              T_NO_PRINT(t.udata);
              if (fmtGFF3 || fmtTable || t.isTranscript()) {
@@ -859,12 +848,14 @@ int main(int argc, char* argv[]) {
 					 t.printGxf(f_out, exonPrinting, tracklabel, NULL, decodeChars);
              }
            }//GFF/GTF output requested
-        } //valid transcript
+        //} //valid transcript
      } //for each rna
      //print the rest of the isolated pseudo/gene/region features not printed yet
      if (f_out && (fmtGFF3 || fmtTable)) {
       while (gfs_i<gdata->gfs.Count()) {
          GffObj& gfst=*(gdata->gfs[gfs_i]);
+         if (TFilters && gfst.isGene() && gfst.children.Count()==0) // gene with no children left, skip it if filters were applied
+        	 { ++gfs_i; continue; }
          if T_PRINTABLE(gfst.udata) { //never printed
            T_NO_PRINT(gfst.udata);
            if (fmtGFF3) {
